unit MRU;

// this example shows the Most Recently Used tabs in red color

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtPage, StdCtrls;

const
  MRUCount = 3;   // three items in the MRU list

type
  TForm1 = class(TForm)
    ExtPageControl1: TExtPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    TabSheet6: TTabSheet;
    TabSheet7: TTabSheet;
    TabSheet8: TTabSheet;
    TabSheet9: TTabSheet;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ExtPageControl1DrawTab(Control: TWinControl; Index: Integer;
      ActiveTab: Boolean; const RectFg, RectBg: TRect;
      State: TOwnerDrawState);
    procedure ExtPageControl1Change(Sender: TObject);
  private
    MRUTabs: TList;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  MRUTabs:= TList.Create;
  // initialization
  for i:= 0 to MRUCount-1 do MRUTabs.Add(Pointer(-1));
  ExtPageControl1Change(ExtPageControl1);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  MRUTabs.Free;
end;

procedure TForm1.ExtPageControl1DrawTab(Control: TWinControl;
  Index: Integer; ActiveTab: Boolean; const RectFg, RectBg: TRect;
  State: TOwnerDrawState);
begin
  with TExtPageControl(Control) do begin
    // check if current tab (index) is in MRU list
    if MRUTabs.IndexOf(Pointer(Index)) >= 0 then begin
      Canvas.Font.Color:= clMaroon;
      Canvas.Font.Style:= [fsBold];
    end;
    Canvas.FillRect(RectBg);
    DefaultDrawTab(Index, RectFg, State);
  end;
end;

procedure TForm1.ExtPageControl1Change(Sender: TObject);
var
  NewMRUEntry: Pointer;
  CurrentPosIsList: Integer;
begin
  with TExtPageControl(Sender) do begin
    NewMRUEntry:= Pointer(ActivePage.PageIndex);
    CurrentPosIsList:= MRUTabs.IndexOf(NewMRUEntry);
    if CurrentPosIsList >= 0 then
      // tab is already in MRU list,
      // we only have to give it the highest priority
      MRUTabs.Move(CurrentPosIsList, 0)
    else begin
      // tab is not in MRU list,
      // we remove the tab with the lowest priority
      MRUTabs.Delete(MRUCount-1);
      // insert the new tab with the highest priority
      MRUTabs.Insert(0, NewMRUEntry);
    end;
    // full graphic update of all tabs is required
    if HandleAllocated then
      InvalidateRect(Handle, nil, False);
  end;
end;

end.
