/*
   Borland C++ Builder TClipboardViewer component
   Version 1.0

   Copyright (C) Marcel de Kogel 1997
   e-mail: dekogel@giganda.komkon.org
   web: http://www.komkon.org/~dekogel

     You are not allowed to distribute this software commercially
     Please, notify me, if you make any changes to this file

   Methods:
     TClipboardViewer(TComponent *Owner) - Constructor
     ~TClipboardViewer() - Destructor
   Properties:
     bool enabled - Enables or disables TClipboardViewer events
   Events:
     TNotifyEvent OnDrawClipboard - Called when the clipboard has changed

   Revisions:
   1.0  30-07-1997  Initial release
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "TClipboardViewer.h"
#define szComponentType	"TClipboardViewer"
//---------------------------------------------------------------------------
static inline TClipboardViewer *ValidCtrCheck()
{
 return new TClipboardViewer(NULL);
}
//---------------------------------------------------------------------------
// Our window procedure
LRESULT CALLBACK _export TClipboardViewerWndProc
	(HWND hwnd,UINT Msg,WPARAM WParam,LPARAM LParam)
{
 // Get a pointer to the TClipboardViewer object this window belongs to
 TClipboardViewer *ClipboardViewer=(TClipboardViewer*)GetWindowLong(hwnd,0);
 // Get the window handle of the next clipboard viewer
 HWND hwndNextClipboardViewer=(HWND)GetWindowLong(hwnd,4);
 switch (Msg)
 {
  case WM_DESTROY:
   // Remove this window from the clipboard viewer chain
   ChangeClipboardChain (hwnd,(HWND)GetWindowLong(hwnd,4));
   break;
  case WM_CHANGECBCHAIN:
  {
   HWND hwndRemoved=(HWND)WParam;
   HWND hwndNext=(HWND)LParam;
   // If window being removed is the one that follows this window, update
   if (hwndNextClipboardViewer==hwndRemoved)
    SetWindowLong(hwnd,4,(LONG)hwndNext);
   // Else pass message to the next viewer
   else if (hwndNextClipboardViewer)
    SendMessage(hwndNextClipboardViewer,Msg,WParam,LParam);
   break;
  }
  case WM_DRAWCLIPBOARD:
   // Pass message to the next viewer
   if (hwndNextClipboardViewer)
    SendMessage(hwndNextClipboardViewer,Msg,WParam,LParam);
   // Call the OnDrawClipboard event
   if (ClipboardViewer->OnDrawClipboard && ClipboardViewer->Enabled)
    ClipboardViewer->OnDrawClipboard(ClipboardViewer);
   break;
  default:
   // Let Windows take care of all the other messages
   return DefWindowProc (hwnd,Msg,WParam,LParam);
 }
 return 0;
}

__fastcall TClipboardViewer::~TClipboardViewer()
{
 // Destroy window
 if (FHandle) DestroyWindow (FHandle);
}

__fastcall TClipboardViewer::TClipboardViewer(TComponent* Owner)
	: TComponent(Owner),
      FHandle(NULL),
      FEnabled(true),
      FOnDrawClipboard(NULL)
{
 // Register TClipboardViewer window class
 WNDCLASS WndClass;
 WndClass.lpszClassName=szComponentType;
 WndClass.hInstance=HInstance;
 WndClass.lpfnWndProc=TClipboardViewerWndProc;
 WndClass.style=0;
 WndClass.lpszMenuName=NULL;
 WndClass.hCursor=NULL;
 WndClass.hIcon=NULL;
 WndClass.hbrBackground=NULL;
 WndClass.cbClsExtra=0;
 WndClass.cbWndExtra=8;
 if (!RegisterClass(&WndClass)) return;
 // Create an invisible TApplication child window
 FHandle=CreateWindow
 		 (szComponentType,		// Window name
		  Name.c_str(),			// Window title
		  WS_CHILD,				// Window style
		  0,					// x-position
		  0,					// y-position
		  0,					// width
		  0,					// height
		  Application->Handle,	// parent window
		  NULL,					// menu handle
		  HInstance,			// application instance
		  0);					// extra parameter
 if (!FHandle)
  return;
 // Put a pointer to this class in the extra window bytes
 SetWindowLong (FHandle,0,(LONG)this);
 // Register this window as a clipboard viewer and put the window handle of
 // the next viewer in the extra window bytes
 SetWindowLong(FHandle,4,(LONG)SetClipboardViewer(FHandle));
}
//---------------------------------------------------------------------------
namespace Tclipboardviewer
{
 void __fastcall Register()
 {
  TComponentClass classes[1] = {__classid(TClipboardViewer)};
  RegisterComponents("MdK", classes, 0);
 }
}
//---------------------------------------------------------------------------
