/*
   Borland C++ Builder TFloatUpDown component
   Version 1.0

   Copyright (C) Marcel de Kogel 1998
   e-mail: dekogel@giganda.komkon.org
   web: http://www.komkon.org/~dekogel

     You are not allowed to distribute this software commercially
     Please, notify me, if you make any changes to this file

   TFloatUpDown is similar to TUpDown, but with the following differences:
   - Property Thousands is not supported
   - The following integer properties have been replaced with double precision
     properties:
	 - Min / FloatMin
     - Max / FloatMax
     - Increment / FloatIncrement
     - Position / FloatPosition
   - The following events have been renamed:
     - OnChanging / OnFloatChanging
     - OnClick / OnFloatClick
     OnFloatClick is called first, immediately followed by OnFloatChanging
     (This doesn't work properly with TUpDown)
   - The property Associate is renamed to FloatAssociate
   TFloatUpDown supports regional settings for floating point number notations

   Revisions:
   1.0  14-09-1998  Initial release
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include <math.h>
#include "TFloatUpDown.h"
//---------------------------------------------------------------------------
static inline TFloatUpDown *ValidCtrCheck()
{
	return new TFloatUpDown(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::CustomUpDownChanging(TObject *Sender,bool &AllowChange)
{
 Position=1;
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::CustomUpDownClick(TObject *Sender,TUDBtnType Button)
{
 if (FOnFloatClick) FOnFloatClick(Sender,Button);
 double CurrentPosition=FFloatPosition;
 if (FFloatAssociate)
 {
  TCustomEdit *CustomEdit=NULL;
  try
  {
   CustomEdit=dynamic_cast<TCustomEdit*>(FFloatAssociate);
  }
  catch (Exception &exception) {};
  if (CustomEdit)
  {
   CurrentPosition=0;
   try
   {
    CurrentPosition=CustomEdit->Text.ToDouble();
   }
   catch (Exception &exception) {};
  }
 }
 double NewPosition;
 if (Button==btNext)
  NewPosition=CurrentPosition+FFloatIncrement;
 else
  NewPosition=CurrentPosition-FFloatIncrement;
 // Round to nearest multiple of FFloatIncrement
 double mod=fmod(NewPosition,FFloatIncrement);
 if (mod)
 {
  if (mod>FFloatIncrement/2)
   NewPosition=ceil(NewPosition/FloatIncrement)*FloatIncrement;
  else
   NewPosition=floor(NewPosition/FloatIncrement)*FloatIncrement;
 }
 if (NewPosition<FFloatMin) NewPosition=FFloatMin;
 if (NewPosition>FFloatMax) NewPosition=FFloatMax;
// Set FloatPosition too, so don't do the following test
// if (NewPosition==CurrentPosition) return;
 if (OnFloatChanging)
 {
  bool AllowChange=true;
  OnFloatChanging(this,AllowChange);
  if (!AllowChange) return;
 }
 SetFloatPosition(NewPosition);
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::SetFloatMin(double FloatMin)
{
 FFloatMin=FloatMin;
 if (FFloatMin>FFloatMax) FFloatMax=FFloatMin;
 if (FFloatPosition<FFloatMin) SetFloatPosition(FFloatMin);
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::SetFloatMax(double FloatMax)
{
 FFloatMax=FloatMax;
 if (FFloatMin>FFloatMax) FFloatMin=FFloatMax;
 if (FFloatPosition>FFloatMax) FFloatPosition=FFloatMax;
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::SetFloatIncrement(double FloatIncrement)
{
 FFloatIncrement=FloatIncrement;
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::SetFloatPosition(double FloatPosition)
{
 if (FloatPosition<FFloatMin) FloatPosition=FFloatMin;
 if (FloatPosition>FFloatMax) FloatPosition=FFloatMax;
 FFloatPosition=FloatPosition;
 if (FFloatAssociate)
 {
  TCustomEdit *CustomEdit=NULL;
  try
  {
   CustomEdit=dynamic_cast<TCustomEdit*>(FFloatAssociate);
  }
  catch (Exception &exception) {};
  if (CustomEdit)
   CustomEdit->Text=AnsiString(FFloatPosition);
 }
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::SetOnFloatChanging(TUDChangingEvent OnFloatChanging)
{
 FOnFloatChanging=OnFloatChanging;
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::SetOnFloatClick(TUDClickEvent OnFloatClick)
{
 FOnFloatClick=OnFloatClick;
}
//---------------------------------------------------------------------------
void __fastcall TFloatUpDown::SetFloatAssociate(TWinControl *FloatAssociate)
{
 // Move and size control
 Associate=FloatAssociate;
 // Don't set Associate to NULL or new TCustomUpDowns controls can't be created
 // anymore (Borland VCL bug)
 Associate=this;
 // Set our own associate variable
 FFloatAssociate=FloatAssociate;
 // Set text in edit control
 SetFloatPosition(FFloatPosition);
}
//---------------------------------------------------------------------------
__fastcall TFloatUpDown::TFloatUpDown(TComponent* Owner)
	: TCustomUpDown(Owner),
	  FFloatMin(0),
      FFloatMax(100),
      FFloatIncrement(1),
	  FFloatPosition(0),
      FOnFloatChanging(NULL)
{
 Min=0;
 Max=2;
 Increment=1;
 Position=1;
 OnChanging=CustomUpDownChanging;
 OnClick=CustomUpDownClick;
}
//---------------------------------------------------------------------------
namespace Tfloatupdown
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(TFloatUpDown)};
		RegisterComponents("MdK", classes, 0);
	}
}
//---------------------------------------------------------------------------
